@@ -1,4 +1,5 @@
 # Copyright 2006-2007 Lukas Lalinsky
+#                2021 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +30,7 @@ from .cdefs import XINE_PARAM_SPEED, XINE_PARAM_GAPLESS_SWITCH, xine_dispose, \
     xine_event_new_queue, xine_event_create_listener_thread
 
 
-class XineHandle(object):
+class XineHandle:
     def __init__(self):
         _xine = xine_new()
         xine_config_load(_xine, xine_get_homedir() + b"/.xine/config")
@@ -70,7 +71,7 @@ class XinePlaylistPlayer(BasePlayer):
     def __init__(self, driver, librarian):
         """May raise PlayerError"""
 
-        super(XinePlaylistPlayer, self).__init__()
+        super().__init__()
         self.name = "xine"
         self.version_info = "xine-lib: " + decode(xine_get_version_string())
         self._volume = 1.0
@@ -273,7 +274,7 @@ class XinePlaylistPlayer(BasePlayer):
         # reset error state
         self.error = False
 
-        current = self._source.current if next_song is None else next_song
+        current = next_song if next_song else (self._source and self._source.current)
 
         # Then, set up the next song.
         self.song = self.info = current
@@ -300,7 +301,7 @@ class XinePlaylistPlayer(BasePlayer):
         self.notify("seekable")
 
     def setup(self, playlist, song, seek_pos):
-        super(XinePlaylistPlayer, self).setup(playlist, song, seek_pos)
+        super().setup(playlist, song, seek_pos)
         # xine's declining to seek so soon after startup; try again in 100ms
         if seek_pos:
             GLib.timeout_add(100, self.seek, seek_pos)
