@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
 # This program is free software; you can redistribute it and/or modify
@@ -7,8 +6,8 @@
 # (at your option) any later version.
 
 import os
-from quodlibet.compat import urlsplit
 import errno
+from urllib.parse import urlsplit
 
 from gi.repository import Gtk, GObject, Gdk, Gio, Pango
 from senf import uri2fsn, fsnative, fsn2text, bytes2fsn
@@ -25,7 +24,6 @@ from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.x import ScrolledWindow, Paned
 from quodlibet.qltk.models import ObjectStore, ObjectTreeStore
 from quodlibet.qltk import Icons
-from quodlibet.compat import xrange
 from quodlibet.util.path import listdir, \
     glib2fsn, xdg_get_user_dirs, get_home_dir, xdg_get_config_home
 from quodlibet.util import connect_obj
@@ -125,27 +123,17 @@ def get_favorites():
         return paths
 
 
-def _get_win_drives():
-    """Returns a list of paths for all available drives e.g. ['C:\\']"""
-
-    assert os.name == "nt"
-    drives = [letter + u":\\" for letter in u"CDEFGHIJKLMNOPQRSTUVWXYZ"]
-    return [d for d in drives if os.path.isdir(d)]
-
-
 def get_drives():
     """A list of accessible drives"""
 
-    if os.name == "nt":
-        return _get_win_drives()
-    else:
-        paths = []
-        for mount in Gio.VolumeMonitor.get().get_mounts():
-            path = mount.get_root().get_path()
-            if path is not None:
-                paths.append(glib2fsn(path))
+    paths = []
+    for mount in Gio.VolumeMonitor.get().get_mounts():
+        path = mount.get_root().get_path()
+        if path is not None:
+            paths.append(glib2fsn(path))
+    if os.name != "nt":
         paths.append("/")
-        return paths
+    return sorted(paths)
 
 
 def parse_gtk_bookmarks(data):
@@ -203,7 +191,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         """
 
         model = ObjectTreeStore()
-        super(DirectoryTree, self).__init__(model=model)
+        super().__init__(model=model)
 
         if initial is not None:
             assert isinstance(initial, fsnative)
@@ -288,12 +276,16 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
     def go_to(self, path_to_go):
         assert isinstance(path_to_go, fsnative)
 
-        # FIXME: what about non-normalized paths?
+        # The path should be normalized in normal situations.
+        # On some systems and special environments (pipenv) there might be
+        # a non-normalized path at least during tests, though.
+        path_to_go = os.path.normpath(path_to_go)
+
         model = self.get_model()
 
         # Find the top level row which has the largest common
         # path with the path we want to go to
-        roots = dict([(p, i) for (i, p) in model.iterrows(None)])
+        roots = dict((p, i) for (i, p) in model.iterrows(None))
         head, tail = path_to_go, fsnative(u"")
         to_find = []
         while head and head not in roots:
@@ -431,7 +423,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         nchildren = model.iter_n_children(iter_)
         last = model.get_path(iter_)
 
-        for i in xrange(nchildren):
+        for i in range(nchildren):
             child = model.iter_nth_child(iter_, i)
             self.expand_row(model.get_path(child), False)
             last = self.__select_children(child, model, selection)
@@ -509,7 +501,7 @@ class FileSelector(Paned):
         folders -- list of shown folders in the directory tree
         """
 
-        super(FileSelector, self).__init__(
+        super().__init__(
             orientation=Gtk.Orientation.VERTICAL)
         self.__filter = filter
 
@@ -522,6 +514,7 @@ class FileSelector(Paned):
 
         model = ObjectStore()
         filelist = AllTreeView(model=model)
+        filelist.connect("draw", self.__restore_scroll_pos_on_draw)
 
         column = TreeViewColumn(title=_("Songs"))
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
@@ -634,6 +627,13 @@ class FileSelector(Paned):
 
         fselect.handler_unblock(self.__sig)
         fselect.emit('changed')
+        self._saved_scroll_pos = filelist.get_vadjustment().get_value()
+
+    def __restore_scroll_pos_on_draw(self, treeview, context):
+        if self._saved_scroll_pos:
+            vadj = treeview.get_vadjustment()
+            vadj.set_value(self._saved_scroll_pos)
+            self._saved_scroll_pos = None
 
 
 def _get_main_folders():
@@ -670,7 +670,7 @@ class MainFileSelector(FileSelector):
 
     def __init__(self, initial=None):
         folders = _get_main_folders()
-        super(MainFileSelector, self).__init__(
+        super().__init__(
             initial, filesel_filter, folders=folders)
 
 
@@ -690,5 +690,5 @@ class MainDirectoryTree(DirectoryTree):
         if main:
             folders += main
 
-        super(MainDirectoryTree, self).__init__(
+        super().__init__(
             initial=initial, folders=folders)
