@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
+#           2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@ ease constructors.
 
 from urllib.request import urlopen
 
-from gi.repository import Gtk, GObject, GLib, Gio, GdkPixbuf
+from gi.repository import Gtk, GObject, GLib, Gio, GdkPixbuf, Gdk
 
 from quodlibet.util.dprint import print_d
 
@@ -140,22 +140,42 @@ class ScrolledWindow(Gtk.ScrolledWindow):
         return Gtk.ScrolledWindow.do_size_allocate(self, alloc)
 
 
+MT = Gdk.ModifierType
+
+
 class Notebook(Gtk.Notebook):
     """A regular gtk.Notebook, except when appending a page, if no
     label is given, the page's 'title' attribute (either a string or
     a widget) is used."""
 
+    _KEY_MODS = (MT.SHIFT_MASK | MT.CONTROL_MASK | MT.MOD1_MASK | MT.MOD2_MASK)
+    """Keyboard modifiers of interest"""
+
     def __init__(self, *args, **kwargs):
-        super(Notebook, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connect("key-press-event", self.__key_pressed)
 
-    def __key_pressed(self, widget, event):
+    def __key_pressed(self, _widget: Gtk.Widget, event: Gdk.Event):
         # alt+X switches to page X
         for i in range(self.get_n_pages()):
             if is_accel(event, "<alt>%d" % (i + 1)):
                 self.set_current_page(i)
-                return True
-        return False
+                return Gdk.EVENT_STOP
+
+        state = event.state & self._KEY_MODS
+        # Use hardware, as Gtk+ seems to special-case tab for itself
+        if event.hardware_keycode == 23:
+            total = self.get_n_pages()
+            current = self.get_current_page()
+            if state == (MT.SHIFT_MASK | MT.CONTROL_MASK | MT.MOD2_MASK):
+                self.set_current_page((current + total - 1) % total)
+                return Gdk.EVENT_STOP
+            elif state == (MT.CONTROL_MASK | MT.MOD2_MASK):
+                self.set_current_page((current + 1) % total)
+                return Gdk.EVENT_STOP
+            else:
+                print_d(f"Unhandled tab key combo: {event.state}")
+        return Gdk.EVENT_PROPAGATE
 
     def do_size_allocate(self, alloc):
         ctx = self.get_style_context()
@@ -204,7 +224,7 @@ class Notebook(Gtk.Notebook):
 
         if not isinstance(label, Gtk.Widget):
             label = Gtk.Label(label=label)
-        super(Notebook, self).append_page(page, label)
+        super().append_page(page, label)
 
 
 def Frame(label, child=None):
@@ -253,7 +273,7 @@ class Align(Gtk.Alignment):
         left_padding = border + left
         right_padding = border + right
 
-        super(Align, self).__init__(xalign=xalign, xscale=xscale,
+        super().__init__(xalign=xalign, xscale=xscale,
             yalign=yalign, yscale=yscale, bottom_padding=bottom_padding,
             top_padding=top_padding, left_padding=left_padding,
             right_padding=right_padding)
@@ -290,10 +310,10 @@ def MenuItem(label, icon_name=None):
 
 def _Button(type_, label, icon_name, size):
     if icon_name is None:
-        return Gtk.Button.new_with_mnemonic(label)
+        return type_.new_with_mnemonic(label)
 
     align = Align(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER)
-    hbox = Gtk.HBox(spacing=2)
+    hbox = Gtk.HBox(spacing=6)
     image = Gtk.Image.new_from_icon_name(icon_name, size)
     hbox.pack_start(image, True, True, 0)
     label = Gtk.Label(label=label)
@@ -322,11 +342,11 @@ def ToggleButton(label, icon_name=None, size=Gtk.IconSize.BUTTON):
     return _Button(Gtk.ToggleButton, label, icon_name, size)
 
 
-class _SmallImageButton(object):
+class _SmallImageButton:
     """A button for images with less padding"""
 
     def __init__(self, **kwargs):
-        super(_SmallImageButton, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         self.set_size_request(26, 26)
         add_css(self, """
@@ -384,7 +404,7 @@ def SymbolicIconImage(name, size, fallbacks=None):
 class CellRendererPixbuf(Gtk.CellRendererPixbuf):
 
     def __init__(self, *args, **kwargs):
-        super(CellRendererPixbuf, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if gtk_version < (3, 16):
             # was deprecated in 3.16 and defaults to True now. Since it was
             # False before force it here so we have the same behavior in all
@@ -422,7 +442,7 @@ class WebImage(Gtk.Image):
             height (int): a height to reserve for the image or -1
         """
 
-        super(WebImage, self).__init__()
+        super().__init__()
 
         self._cancel = Cancellable()
         call_async(self._fetch_image, self._cancel, self._finished, (url,))
@@ -460,7 +480,7 @@ class HighlightToggleButton(Gtk.ToggleButton):
     """A ToggleButton which changes the foreground color when active"""
 
     def __init__(self, *args, **kwargs):
-        super(HighlightToggleButton, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._provider = None
         self._color = ""
         self._dummy = Gtk.ToggleButton()
